// -*- coding: iso-8859-1-unix -*-

/*
** scs/core/IComponent.h
*/

#ifndef TECGRAF_SCS_ICOMPONENT_HPP
#define TECGRAF_SCS_ICOMPONENT_HPP

#include <scs/private/config.hpp>
#include <scs/ComponentContext.hpp>
#include <stubs/scs.h>

#include <CORBA.h>
#include <map>

namespace scs {
  namespace core {
    class ComponentContext;

    //! \brief Implementacao de servant da interface scs.core.IComponent.
    class SCS_DECL IComponentImpl : public virtual POA_scs::core::IComponent 
                         , public virtual PortableServer::RefCountServantBase
    {
      private:
        ComponentContext* component;
      public:
        /**
         * \brief Construtor
         *
         * \param component Referencia para uma instancia de ComponentContext
         */
        IComponentImpl(ComponentContext& component);
        //! Destrutor
        ~IComponentImpl();

        /**
         * \brief Funo para ativar o componente
         *
         * Essa funo  vazia nessa implementao
         */
        void startup();
        
        /**
         * \brief Funo para desativar o componente
         *
         * Essa funo  vazia nessa implementao
         */
        void shutdown();

        /**
         * \brief Retorna faceta
         *
         * \param facet_interface Interface da faceta
         *
         * Essa funo retorna a faceta de interface facet_interface de
         * ComponentContext. Essa funo serializa o acesso ao estado
         * das facets no ComponentContext com ComponentContext::facet_lock.
         */
        CORBA::Object_ptr getFacet(const char* facet_interface);
        
        /**
         * \brief Retorna faceta
         *
         * \param facet Nome da faceta
         *
         * Essa funo retorna a faceta com nome facet de
         * ComponentContext. Essa funo serializa o acesso ao estado
         * das facets no ComponentContext com
         * ComponentContext::facet_lock.
         */
        CORBA::Object_ptr getFacetByName(const char* facet);
        
        //! Retorna o identificador do componente
        ComponentId* getComponentId();
      };
  }
}

#endif

