// -*- coding: iso-8859-1-unix -*-

/*
** IMetaInterfaceImpl.h
*/

#ifndef TECGRAF_SCS_IMETAINTERFACEIMPL_HPP
#define TECGRAF_SCS_IMETAINTERFACEIMPL_HPP

#include <scs/private/config.hpp>
#include <scs/ComponentContext.hpp>
#include <stubs/scs.h>

#include <CORBA.h>
#include <map>

namespace scs {
  namespace core {
    //! Implementacao do servant da interface scs.core.IMetaInterface
    class SCS_DECL IMetaInterfaceImpl : virtual public POA_scs::core::IMetaInterface
                             , virtual public PortableServer::RefCountServantBase
    {
      ComponentContext* component;
    public:
      /**
       * \brief Construtor
       *
       * \param component Referencia para uma instancia de ComponentContext
       */
      IMetaInterfaceImpl(ComponentContext& component);
      //! Destrutor
      ~IMetaInterfaceImpl();

      /**
       * \brief Retorna as descricoes de todas as facetas do componente
       *
       * Essa funo possui complexidade O(n), aonde n e a quantidade de facetas
       * do componente
       */
      FacetDescriptions* getFacets();

      /**
       * \brief Retorna as descricoes das facetas com nomes especificos do componente
       *
       * \param names Lista de nomes das facetas
       *
       * Essa funcao possui complexidade O(log m*n), aonde m e a quantidade de 
       * de facetas no componente e n e o tamanho da lista de nomes passada
       * como argumento.
       */
      FacetDescriptions* getFacetsByName(const NameList&  names);

      //! Retorna as descricoes de todos os receptaculos do componente
      ReceptacleDescriptions* getReceptacles();

      /**
       * \brief Retorna as descricoes dos receptaculos com nomes especificos do componente
       *
       * \param names Lista de nomes dos receptaculos
       */
      ReceptacleDescriptions* getReceptaclesByName(const NameList &  names);
    };
  }
}

#endif 

