// -*- coding: iso-8859-1-unix -*-

/*
** IReceptacles.h
*/

#ifndef TECGRAF_SCS_IRECEPTACLES_HPP
#define TECGRAF_SCS_IRECEPTACLES_HPP

#include <scs/private/config.hpp>
#include <scs/ComponentContext.hpp>
#include <stubs/scs.h>

#include <CORBA.h>
#include <cstdlib>

namespace scs {
  namespace core {
    //! Implementacao do servant da interface scs.core.IReceptacles
    class SCS_DECL IReceptaclesImpl : virtual public POA_scs::core::IReceptacles
                           , virtual public PortableServer::RefCountServantBase
    {
    private:
      ComponentContext* component;
      int connectionCounter;
      std::size_t connectionLimit;

    public:
      /**
       * \brief Construtor
       *
       * \param component Referencia para uma instancia de ComponentContext
       */
      IReceptaclesImpl(ComponentContext& component);
      //! Destrutor
      ~IReceptaclesImpl();
      /**
       * \brief Implementacao para conectar um objeto a um receptaculo
       *
       * \param receptacle O nome do receptaculo
       * \param obj Objeto a se conectar
       *
       * \return O identificador da conexao criada.
       *
       * A complexidade desta funcao e O(n+log n+log m), onde n e o numero de
       * receptaculos no componente, e m e a media de conexoes por
       * receptaculo.
       */
      ConnectionId connect (const char* receptacle, CORBA::Object_ptr obj);

      /**
       * \brief Implementacao para fazer uma desconexao
       *
       * \param id Identificador da conexao
       *
       * A complexidade desta funcao e O(n*m), onde n e o numero de
       * receptaculos no componente, e m e a media de conexoes por
       * receptaculo.
       */
      void disconnect (ConnectionId id);

      /**
       * \brief Retorna as descricoes de todas as conexoes de um receptaculo
       *
       * \param receptacle Nome do receptaculo
       *
       * A complexidade desta funcao e O(log m + n), onde n e o numero
       * de conexoes do receptaculo e m e o numero de receptaculos do
       * componente
       */
      ConnectionDescriptions* getConnections (const char* receptacle);
    };
  }
}

#endif

