// -*- coding: iso-8859-1-unix -*-

/*
** IReceptacles.cpp
*/

#include <scs/IReceptacles.hpp>

#include <cstdlib>
#include <limits>

namespace scs {
  namespace core {
    IReceptaclesImpl::IReceptaclesImpl(ComponentContext& context)
      : component(&context), connectionCounter(0)
      , connectionLimit((std::numeric_limits<int>::max)())
    {
    }

    IReceptaclesImpl::~IReceptaclesImpl() {
    }

    ConnectionId IReceptaclesImpl::connect(
      const char* receptacle, 
      CORBA::Object_ptr obj)
    {
#ifdef SCS_THREADING_ENABLED
      ComponentContext::receptacles_lock lock(*component);
#endif
      std::map<std::string, Receptacle>&
        receptacles = component->getMapReceptacles();
      std::map<std::string, Receptacle>::iterator
        receptacle_iterator = receptacles.find(receptacle);       
      if (receptacle_iterator == receptacles.end())
        throw InvalidName();

      if (receptacle_iterator->second.multiplexed
          && !receptacle_iterator->second.connections.empty())
        throw AlreadyConnected();

      std::size_t connections = 0;
      for(std::map<std::string, Receptacle>::const_iterator
            receptacles_iterator = component->getMapReceptacles().begin()
            , receptacles_iterator_end = component->getMapReceptacles().end()
            ;receptacles_iterator != receptacles_iterator_end
            ;++receptacles_iterator)
        connections += receptacles_iterator->second.connections.size();

      if (connections >= connectionLimit)
        throw ExceededConnectionLimit();

      if (!obj->_is_a(receptacle_iterator->second.interface_name.c_str()))
        throw InvalidConnection();

      CORBA::Object_var obj_var = obj;	
      receptacle_iterator->second.connections.insert
        (std::make_pair(++component->current_connection_id, obj_var));
      return component->current_connection_id;
    }

    void IReceptaclesImpl::disconnect (ConnectionId id)
    {
#ifdef SCS_THREADING_ENABLED
      ComponentContext::receptacles_lock lock(*component);
#endif
      if (id < 0)
        throw InvalidConnection();

      std::map<std::string, Receptacle>::iterator
        receptacle_iterator = component->getMapReceptacles().begin()
        , receptacle_iterator_end = component->getMapReceptacles().end();
      std::map<ConnectionId, CORBA::Object_var>::iterator
        connection_iterator
        , connection_iterator_end;
      bool found = false;
      while(!found && receptacle_iterator != receptacle_iterator_end)
      {
        connection_iterator = receptacle_iterator->second.connections.begin();
        connection_iterator_end = receptacle_iterator->second.connections.end();
        while(!found && connection_iterator != connection_iterator_end)
        {
          if(connection_iterator->first == id)
          {
            found = true;
            break;
          }
          ++connection_iterator;
        }
        if(found)
          break;
        ++receptacle_iterator;
      }

      assert(found?receptacle_iterator != receptacle_iterator_end
             :receptacle_iterator == receptacle_iterator_end);

      if(!found)
        throw NoConnection();

      receptacle_iterator->second.connections.erase(connection_iterator);
    }

    ConnectionDescriptions* IReceptaclesImpl::getConnections(
      const char* receptacle)
    {
#ifdef SCS_THREADING_ENABLED
      ComponentContext::receptacles_lock lock(*component);
#endif
      std::map<std::string, Receptacle>&
        receptacles = component->getMapReceptacles();
      std::map<std::string, Receptacle>::iterator
        receptacle_iterator = receptacles.find(receptacle);       
      if (receptacle_iterator == receptacles.end())
        throw InvalidName();
      ConnectionDescriptions_var connections
        (new ConnectionDescriptions(receptacle_iterator->second.connections.size()));
      connections->length(receptacle_iterator->second.connections.size());
      CORBA::ULong i = 0;
      for(std::map<ConnectionId, CORBA::Object_var>::const_iterator
            Iter = receptacle_iterator->second.connections.begin()
            , EndIter = receptacle_iterator->second.connections.end()
            ;Iter != EndIter
            ;++Iter, ++i)
      {
        ConnectionDescription d;
        d.id = Iter->first;
        d.objref = Iter->second;
        connections[i] = d;
      }
      return connections._retn();
    }
  }
}
