/*
* ReceptaclesTestSuite.cpp
*/

#ifndef RECEPTACLES_TESTSUITE_H
#define RECEPTACLES_TESTSUITE_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fstream>
#include <cxxtest/TestSuite.h>
#include <ComponentBuilder.hpp>

class ReceptaclesTestSuite: public CxxTest::TestSuite {
  private:
    CORBA::ORB* orb;
    PortableServer::POA_var poa;
    scs::core::IComponent_var component;
    scs::core::IComponent_var component2;
    scs::core::ComponentBuilder* componentBuilder;
    scs::core::ComponentContext* context;
    scs::core::ComponentContext* context2;
    scs::core::IReceptacles_var rec;
    std::string recName;
    bool err;
    int connId;

    void fillComponentId(scs::core::ComponentId& id) {
      id.name = "ReceptaclesTestSuiteComponent";
      id.major_version = '1';
      id.minor_version = '0';
      id.patch_version = '0';
      id.platform_spec = "none";
    }

  public:
    ReceptaclesTestSuite() {
      try {
        recName = "MyReceptacle";
        err = false;

        const char* argv[] = {};
        int argc = 0;
        orb = CORBA::ORB_init(argc, (char**)argv);
        CORBA::Object_var poa_obj = orb->resolve_initial_references("RootPOA");
        poa = PortableServer::POA::_narrow(poa_obj);
        PortableServer::POAManager_var poa_manager = poa->the_POAManager();
        poa_manager->activate();

        componentBuilder = new scs::core::ComponentBuilder(orb, poa);

        scs::core::ComponentId id;
        fillComponentId(id);

        std::list<scs::core::ExtendedFacetDescription> extFacets;
        context = componentBuilder->newComponent(extFacets, id);
        TS_ASSERT(NULL != context);
        component = context->getIComponent();

        std::list<scs::core::ReceptacleDescription> receptDescs;
        scs::core::ReceptacleDescription desc;
        desc.name = recName.c_str();
        desc.interface_name = "IDL:scs/core/IComponent:1.0";
        desc.is_multiplex = false;
        receptDescs.push_front(desc);
        context2 = componentBuilder->newComponent(extFacets, receptDescs, id);
        TS_ASSERT(NULL != context2);
        component2 = context2->getIComponent();

        rec = scs::core::IReceptacles::_narrow(component2->getFacet("IDL:scs/core/IReceptacles:1.0"));
      }
      catch ( const char* errmsg ) {
        TS_FAIL( errmsg );
      }
    }

    ~ReceptaclesTestSuite() {
      try {
        delete context;
        delete context2;
        delete componentBuilder;
      }
      catch (const char* errmsg ) {
        TS_FAIL(errmsg);
      }
    }

    void setUP() {
    }

    void tearDown() {
    }

    void testConnect() {
      try {
        // testa nome errado e faceta certa
        rec->connect("WrongReceptacle", component);
      }
      catch (scs::core::InvalidName& e) {
        err = true;
      }
      if (err == false) {
        TS_FAIL("ERRO: Foi realizada uma conexo a um receptculo inexistente!");
      }
      err = false;
      try {
        // testa nome certo e faceta errada
        rec->connect(recName.c_str(), rec);
      }
      catch (scs::core::InvalidConnection& e) {
        err = true;
      }
      if (err == false) {
        TS_FAIL("ERRO: Foi realizada uma conexo a uma faceta de tipo diferente da especificada!");
      }
      err = false;
      try {
        // conecta
        connId = rec->connect(recName.c_str(), component);
      }
      catch (const char* errmsg) {
        std::string temp(errmsg);
        TS_FAIL("ERRO AO REALIZAR CONEXO LEGTIMA: " + temp);
      }
      try {
        // tenta conectar novamente para checar erro AlreadyConnected
        rec->connect(recName.c_str(), component);
      }
      catch (scs::core::AlreadyConnected& e) {
        err = true;
      }
      if (err == false) {
        TS_FAIL("ERRO: Foi realizada uma conexo a um receptculo j conectado!");
      }
      err = false;
    }

    void testGetConnections() {
      try {
        // tenta obter conexoes de um receptaculo nao existente
        rec->getConnections("WrongReceptacle");
      }
      catch (scs::core::InvalidName& e) {
        err = true;
      }
      if (err == false) {
        TS_FAIL("ERRO: Foram obtidas conexes de um receptculo inexistente!");
      }
      err = false;
      try {
        // obtem conexoes
        rec->getConnections(recName.c_str());
      }
      catch (const char* errmsg) {
        std::string temp(errmsg);
        TS_FAIL("ERRO AO OBTER CONEXES: " + temp);
      }
    }

    void testDisconnect() {
      try {
        // tenta desconectar com um id de conexao inexistente
        rec->disconnect(9999);
      }
      catch (scs::core::NoConnection& e) {
        err = true;
      }
      if (err == false) {
        TS_FAIL("ERRO: Foi desfeita uma conexo inexistente!");
      }
      err = false;
      try {
        // desconecta
        rec->disconnect(connId);
      }
      catch (const char* errmsg) {
        std::string temp(errmsg);
        TS_FAIL("ERRO AO DESCONECTAR: " + temp);
      }
      try {
        // tenta desconectar com o id que acabou de desconectar
        rec->disconnect(connId);
      }
      catch (scs::core::NoConnection& e) {
        err = true;
      }
      if (err == false) {
        TS_FAIL("ERRO: Foi desfeita uma conexo inexistente!");
      }
      err = false;
    }
};

#endif

