
#include <scs/ComponentContext.hpp>
#include <hello.h>

#include <iostream>
#include <cstdlib>

std::size_t hellos = 0;

struct HelloServant : POA_scs::test::helloworld::Hello
{
  HelloServant()
  {
    ++hellos;
  }
  ~HelloServant()
  {
    --hellos;
  }

  void sayHello()
  {
    std::cout << "Hello" << std::endl;
  }
};

struct ServantLocator : public virtual POA_PortableServer::ServantLocator
{
  PortableServer::ServantBase* preinvoke (PortableServer::ObjectId const& oid, PortableServer::POA* poa
                                          , const char* operation, void*& cookie)
  {
    return new HelloServant;
  }
  void postinvoke(PortableServer::ObjectId const& oid, PortableServer::POA* poa, const char* operation
                  , void* cookie, PortableServer::ServantBase* servant)
  {
    delete servant;
  }
};

int main(int argc, char** argv)
{
  {
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "");
  
    CORBA::Object_var root_poa_obj = orb->resolve_initial_references("RootPOA");
    if(!CORBA::is_nil(root_poa_obj))
    {
      PortableServer::POA_var root_poa = PortableServer::POA::_narrow(root_poa_obj);
      {
        PortableServer::POAManager_var mgr = root_poa->the_POAManager();
        mgr->activate();
      }

      PortableServer::POA_var child_poa;
      {
        CORBA::PolicyList policy_list;
        policy_list.length(2);
        
        policy_list[0] = root_poa->create_servant_retention_policy(PortableServer::NON_RETAIN);
        policy_list[1] = root_poa->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
        
        child_poa = root_poa->create_POA("EvictorDesignTest"
                                       , PortableServer::POAManager::_nil()
                                       , policy_list);
        PortableServer::POAManager_var mgr = child_poa->the_POAManager();
        mgr->activate();
      }

      ServantLocator locator;
      PortableServer::ServantManager_var servant_manager(locator._this());
      try
      {
        child_poa->set_servant_manager(servant_manager);
      }
      catch(CORBA::Exception* e)
      {
        delete e;
        std::abort();
      }

      scs::core::ComponentId component_id = {"MyComponent", 1, 0, 0, ""};

      scs::core::ComponentContext context(orb, root_poa, component_id);
      scs::core::IComponent_var component = context.getIComponent();
      assert(!CORBA::is_nil(component));

      assert(hellos == 0);
      CORBA::Object_var hello_facet = child_poa->create_reference("IDL:scs/test/helloworld/Hello:1.0");
      context.addFacet("HelloFacet", "IHello", hello_facet);
      assert(hellos == 0);

      {
        CORBA::Object_var facet_obj = component->getFacetByName("HelloFacet");
        assert(!CORBA::is_nil(facet_obj));
        scs::test::helloworld::Hello_var hello_facet = scs::test::helloworld::Hello::_narrow(facet_obj);
        assert(hellos == 0);
        hello_facet->sayHello();
        assert(hellos == 0);
        assert(!CORBA::is_nil(hello_facet));
      }
      {
        CORBA::Object_var facet_obj = component->getFacet("IHello");
        assert(!CORBA::is_nil(facet_obj));
        scs::test::helloworld::Hello_var hello_facet = scs::test::helloworld::Hello::_narrow(facet_obj);
        assert(hellos == 0);
        hello_facet->sayHello();
        assert(hellos == 0);
        assert(!CORBA::is_nil(hello_facet));
      }
    }
    else
      return -1;
    assert(hellos == 0);
  }
}
