
#include <scs/ComponentContext.hpp>
#include <cstring>

int main(int argc, char** argv)
{
  CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "");
  
  CORBA::Object_var poa_obj = orb->resolve_initial_references("RootPOA");
  if(!CORBA::is_nil(poa_obj))
  {
    PortableServer::POA_var poa = PortableServer::POA::_narrow(poa_obj);
    PortableServer::POAManager_var mgr = poa->the_POAManager();
    mgr->activate();
    scs::core::ComponentId component_id = {"MyComponent", 1, 0, 0, ""};

    scs::core::ComponentContext context(orb, poa, component_id);
    scs::core::IComponent_var component = context.getIComponent();
    assert(!CORBA::is_nil(component));
    
    {
      CORBA::Object_var meta_interface_obj = component->getFacet("IDL:scs/core/IMetaInterface:1.0");
      assert(!CORBA::is_nil(meta_interface_obj));
      scs::core::IMetaInterface_var meta_interface_facet = scs::core::IMetaInterface::_narrow(meta_interface_obj);
      assert(!CORBA::is_nil(meta_interface_facet));

      {
        scs::core::NameList_var component_name_list
          = new scs::core::NameList(1);
        component_name_list->length(1);
        CORBA::String_var component_name = CORBA::string_dup("IComponent");
        (*component_name_list)[0] = component_name;

        scs::core::FacetDescriptions_var descriptions = meta_interface_facet->getFacetsByName(component_name_list);
        assert(descriptions->length() == 1);

        assert(!std::strcmp((*descriptions)[0].interface_name.in(), "IDL:scs/core/IComponent:1.0"));
        assert(!std::strcmp((*descriptions)[0].name.in(), "IComponent"));
        assert(!CORBA::is_nil((*descriptions)[0].facet_ref.in()));
      }

      {
        scs::core::NameList_var receptacles_name_list
          = new scs::core::NameList(1);
        receptacles_name_list->length(1);
        CORBA::String_var receptacles_name = CORBA::string_dup("IReceptacles");
        (*receptacles_name_list)[0] = receptacles_name;

        scs::core::FacetDescriptions_var descriptions = meta_interface_facet->getFacetsByName(receptacles_name_list);
        assert(descriptions->length() == 1);

        assert(!std::strcmp((*descriptions)[0].interface_name.in(), "IDL:scs/core/IReceptacles:1.0"));
        assert(!std::strcmp((*descriptions)[0].name.in(), "IReceptacles"));
        assert(!CORBA::is_nil((*descriptions)[0].facet_ref.in()));
      }

      {
        scs::core::NameList_var meta_interface_name_list
          = new scs::core::NameList(1);
        meta_interface_name_list->length(1);
        CORBA::String_var meta_interface_name = CORBA::string_dup("IMetaInterface");
        (*meta_interface_name_list)[0] = meta_interface_name;

        scs::core::FacetDescriptions_var descriptions = meta_interface_facet->getFacetsByName(meta_interface_name_list);
        assert(descriptions->length() == 1);

        assert(!std::strcmp((*descriptions)[0].interface_name.in(), "IDL:scs/core/IMetaInterface:1.0"));
        assert(!std::strcmp((*descriptions)[0].name.in(), "IMetaInterface"));
        assert(!CORBA::is_nil((*descriptions)[0].facet_ref.in()));
      }
    }
  }
  else
    return -1;
}
