
#include <scs/ComponentContext.hpp>
#include <cstring>

int main(int argc, char** argv)
{
  CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "");
  
  CORBA::Object_var poa_obj = orb->resolve_initial_references("RootPOA");
  if(!CORBA::is_nil(poa_obj))
  {
    PortableServer::POA_var poa = PortableServer::POA::_narrow(poa_obj);
    PortableServer::POAManager_var mgr = poa->the_POAManager();
    mgr->activate();
    scs::core::ComponentId component_id = {"MyComponent", 1, 0, 0, ""};

    scs::core::ComponentContext context(orb, poa, component_id);
    scs::core::IComponent_var component = context.getIComponent();
    assert(!CORBA::is_nil(component));
    
    {
      CORBA::Object_var meta_interface_obj = component->getFacet("IDL:scs/core/IMetaInterface:1.0");
      assert(!CORBA::is_nil(meta_interface_obj));
      scs::core::IMetaInterface_var meta_interface_facet = scs::core::IMetaInterface::_narrow(meta_interface_obj);
      assert(!CORBA::is_nil(meta_interface_facet));

      scs::core::FacetDescriptions_var descriptions = meta_interface_facet->getFacets();
      assert(descriptions->length() == 3);

      CORBA::ULong component_index = 0
        , receptacles_index = 0
        , meta_interface_index = 0;
      bool component_found = false
        , receptacles_found = false
        , meta_interface_found = false;
      for(CORBA::ULong i = 0; i != 3; ++i)
      {
        if(!std::strcmp((*descriptions)[i].interface_name.in(), "IDL:scs/core/IComponent:1.0"))
        {
          component_index = i;
          assert(!component_found);
          component_found = true;
        }
        else if(!std::strcmp((*descriptions)[i].interface_name.in(), "IDL:scs/core/IReceptacles:1.0"))
        {
          receptacles_index = i;
          assert(!receptacles_found);
          receptacles_found = true;
        }
        if(!std::strcmp((*descriptions)[i].interface_name.in(), "IDL:scs/core/IMetaInterface:1.0"))
        {
          meta_interface_index = i;
          assert(!meta_interface_found);
          meta_interface_found = true;
        }
      }

      assert(component_found && receptacles_found && meta_interface_found);
      
      assert(!std::strcmp((*descriptions)[component_index].interface_name.in(), "IDL:scs/core/IComponent:1.0"));
      assert(!std::strcmp((*descriptions)[component_index].name.in(), "IComponent"));
      assert(!CORBA::is_nil((*descriptions)[component_index].facet_ref.in()));
      assert(!std::strcmp((*descriptions)[receptacles_index].interface_name.in(), "IDL:scs/core/IReceptacles:1.0"));
      assert(!std::strcmp((*descriptions)[receptacles_index].name.in(), "IReceptacles"));
      assert(!CORBA::is_nil((*descriptions)[receptacles_index].facet_ref.in()));
      assert(!std::strcmp((*descriptions)[meta_interface_index].interface_name.in(), "IDL:scs/core/IMetaInterface:1.0"));
      assert(!std::strcmp((*descriptions)[meta_interface_index].name.in(), "IMetaInterface"));
      assert(!CORBA::is_nil((*descriptions)[meta_interface_index].facet_ref.in()));
    }
  }
  else
    return -1;
}
