
#include <scs/ComponentContext.hpp>
#include <hello.h>

#include <iostream>
#include <cstdlib>

#include <mico/os-thread.h>

struct HelloServant : POA_scs::test::helloworld::Hello
{
  void sayHello()
  {
  }
};

struct component_run_thread : MICOMT::Thread
{
  component_run_thread(scs::core::ComponentContext& context)
    : context(&context) {}

  void _run(void*)
  {
    int facet_number = *static_cast<int*>(_arg);
    std::cout << "Running thread " << facet_number << std::endl;
    delete static_cast<int*>(_arg);

    PortableServer::POA_var poa = context->getPOA();
    char name[512]
      , interface_name[512];
    sprintf(name, "Hello%d", facet_number);
    sprintf(interface_name, "IOR:scs/test/helloworld/Hello%d", facet_number);
    context->addFacet(name, interface_name, new HelloServant);

    scs::core::IComponent_var component = context->getIComponent();
    {
      CORBA::Object_var facet_obj = component->getFacetByName(name);
      assert(!CORBA::is_nil(facet_obj));
      scs::test::helloworld::Hello_var hello_facet = scs::test::helloworld::Hello::_narrow(facet_obj);
      assert(!CORBA::is_nil(hello_facet));
    }
    {
      CORBA::Object_var facet_obj = component->getFacet(interface_name);
      assert(!CORBA::is_nil(facet_obj));
      scs::test::helloworld::Hello_var hello_facet = scs::test::helloworld::Hello::_narrow(facet_obj);
      assert(!CORBA::is_nil(hello_facet));
    }
  }

  scs::core::ComponentContext* context;
};

int main(int argc, char** argv)
{
  {
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "");
  
    CORBA::Object_var root_poa_obj = orb->resolve_initial_references("RootPOA");
    if(!CORBA::is_nil(root_poa_obj))
    {
      PortableServer::POA_var root_poa = PortableServer::POA::_narrow(root_poa_obj);
      {
        PortableServer::POAManager_var mgr = root_poa->the_POAManager();
        mgr->activate();
      }

      PortableServer::POA_var child_poa;
      {
        CORBA::PolicyList policy_list;
        policy_list.length(1);
        
        policy_list[0] = root_poa->create_thread_policy(PortableServer::ORB_CTRL_MODEL);
        
        child_poa = root_poa->create_POA("Multithreaded"
                                       , PortableServer::POAManager::_nil()
                                       , policy_list);
        PortableServer::POAManager_var mgr = child_poa->the_POAManager();
        mgr->activate();
      }

      scs::core::ComponentId component_id = {"MyComponent", 1, 0, 0, ""};

      scs::core::ComponentContext context(orb, child_poa, component_id);
      scs::core::IComponent_var component = context.getIComponent();
      assert(!CORBA::is_nil(component));

      std::vector<component_run_thread*> threads;
      const int max_threads = 200;
      for(int i = 0; i != max_threads; i++)
      {
        threads.push_back(new component_run_thread(context));
        int* integer = new int(i);
        threads.back()->start(integer);
      }
      
      for(int i = 0; i != max_threads; i++)
      {
        threads[i]->wait();
        //delete threads[i];
      }

      std::cout << "Test finished" << std::endl;
    }
    else
      return -1;
  }
}
