
#include <scs/ComponentContext.hpp>
#ifndef SCS_ORBIX
#  include <hello.h>
#else
#  include <cstdio>
#  include <helloS.hh>
#endif
#include <memory>
#include <cstdlib>

bool servant_error_destroyed = false;
bool servant_ok_destroyed = false;
std::size_t ref_error_count = 1, ref_ok_count = 1;

struct HelloServant_error : virtual POA_scs::test::helloworld::Hello
{
  void _add_ref ()
  {
    std::cout << "_add_ref " << ++ref_error_count << std::endl;
  }
  void _remove_ref ()
  {
    std::cout << "_remove_ref " << --ref_error_count << std::endl;
    if(!ref_error_count)
      delete this;
  }

  void sayHello()
#ifdef SCS_ORBIX
    throw(CORBA::SystemException)
#endif
  {
    std::cout << "sayHello for wrong Servant" << std::endl;
    std::abort();
  }

  HelloServant_error() {}

  ~HelloServant_error()
  {
    ::servant_error_destroyed = true;
  }
};

struct HelloServant : virtual POA_scs::test::helloworld::Hello
{
  void _add_ref ()
  {
    std::cout << "_add_ref " << ++ref_ok_count << std::endl;
  }
  void _remove_ref ()
  {
    std::cout << "_remove_ref " << --ref_ok_count << std::endl;
    if(!ref_ok_count)
      delete this;
  }

  void sayHello()
#ifdef SCS_ORBIX
    throw(CORBA::SystemException)
#endif
  {
  }

  HelloServant() {}

  ~HelloServant()
  {
    ::servant_ok_destroyed = true;
  }
};

int main(int argc, char** argv)
{
  try
  {
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "");
  
    CORBA::Object_var poa_obj = orb->resolve_initial_references("RootPOA");
    if(!CORBA::is_nil(poa_obj))
    {
      PortableServer::POA_var poa = PortableServer::POA::_narrow(poa_obj);
      PortableServer::POAManager_var mgr = poa->the_POAManager();
      mgr->activate();
      scs::core::ComponentId component_id = {"MyComponent", 1, 0, 0, ""};

      scs::core::ComponentContext context(orb, poa, component_id);
      scs::core::IComponent_var component = context.getIComponent();
      assert(!CORBA::is_nil(component));
    
      std::auto_ptr<PortableServer::ServantBase> hello_error ( new HelloServant_error );
      PortableServer::ServantBase* hello_error_raw = hello_error.get();
      context.addFacet("HelloFacet", "IHello", hello_error);
      hello_error_raw->_remove_ref();
      assert(hello_error.get() == 0);
      {
        CORBA::Object_var facet_obj = component->getFacetByName("HelloFacet");
        assert(!CORBA::is_nil(facet_obj));
        scs::test::helloworld::Hello_var hello_facet = scs::test::helloworld::Hello::_narrow(facet_obj);
        assert(!CORBA::is_nil(hello_facet));
      }
      {
        CORBA::Object_var facet_obj = component->getFacet("IHello");
        assert(!CORBA::is_nil(facet_obj));
        scs::test::helloworld::Hello_var hello_facet = scs::test::helloworld::Hello::_narrow(facet_obj);
        assert(!CORBA::is_nil(hello_facet));
      }

      std::auto_ptr<PortableServer::ServantBase> hello_ok ( new HelloServant );
      PortableServer::ServantBase* hello_ok_raw = hello_ok.get();
      context.updateFacet("HelloFacet", hello_ok);
      hello_ok_raw->_remove_ref();
      assert(hello_ok.get() == 0);
      
      {
        CORBA::Object_var facet_obj = component->getFacetByName("HelloFacet");
        assert(!CORBA::is_nil(facet_obj));
        scs::test::helloworld::Hello_var hello_facet = scs::test::helloworld::Hello::_narrow(facet_obj);
        assert(!CORBA::is_nil(hello_facet));
      }
      {
        CORBA::Object_var facet_obj = component->getFacet("IHello");
        assert(!CORBA::is_nil(facet_obj));
        scs::test::helloworld::Hello_var hello_facet = scs::test::helloworld::Hello::_narrow(facet_obj);
        hello_facet->sayHello();
        assert(!CORBA::is_nil(hello_facet));
      }

      {
        CORBA::Object_var facet_obj = component->getFacet("IHello");
        context.removeFacet("HelloFacet");
      }
    }
    else
      return -1;

    orb->destroy();
  }
  catch(std::exception const& e)
  {
    std::cout << "Exception: " << e.what() << std::endl;
    throw;
  }
  catch(...)
  {
    std::cout << "Some other exception" << std::endl;
    throw;
  }
  std::cout << "Reference count: " << ref_error_count << std::endl;
  std::cout << "Reference count: " << ref_ok_count << std::endl;
  assert(servant_error_destroyed);
  assert(servant_ok_destroyed);
  assert(ref_error_count == 0);
  assert(ref_ok_count == 0);
}
